from pathlib import Path
import requests
import logging
from ml_pipeline import config


logger = logging.getLogger(__name__)

def download(data_path: Path, force=False):

    urls = {
        'train' : 'https://pjreddie.com/media/files/mnist_train.csv',
        'test' : 'https://pjreddie.com/media/files/mnist_test.csv'
    }
    for dataset, url in urls.items():
        filename = data_path / url.split('/')[-1]
        if filename.exists() and not force:
            logger.info(f'file exists {filename} (set force to overwrite)')
            continue
        logger.info(f'downloading {dataset} {url}')
        response = requests.get(url)
        if response.status_code == 200:
            with open(filename, 'wb') as file:
                file.write(response.content)
            logger.info(f'file downloaded {filename}')
        else:
            logger.info(f'failed to download file {filename}')
